# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sjtop']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=0.24.1', 'shioaji>=0.3.3-alpha.3,<0.4.0', 'textual>=0.1.12,<0.2.0']

entry_points = \
{'console_scripts': ['sjtop = sjtop.__main__:main']}

setup_kwargs = {
    'name': 'sjtop',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'Yvictor',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
